package com.css.simulation.resource.algorithm.serviceImpl;

import api.common.pojo.common.ResponseBodyVO;
import api.common.pojo.constants.DictConstants;
import api.common.pojo.param.MinioParameter;
import api.common.pojo.param.algorithm.AlgorithmParameter;
import api.common.pojo.param.project.SimulationManualProjectParam;
import api.common.pojo.param.system.DictParam;
import api.common.pojo.po.algorithm.AlgorithmPO;
import api.common.pojo.vo.algorithm.AlgorithmVO;
import api.common.pojo.vo.algorithm.RunningProjectVO;
import api.common.util.*;
import com.css.simulation.resource.algorithm.mapper.AlgorithmMapper;
import com.css.simulation.resource.algorithm.service.AlgorithmService;
import com.css.simulation.resource.common.utils.AuthUtil;
import com.css.simulation.resource.common.utils.PageUtil;
import com.css.simulation.resource.common.utils.PoUtil;
import com.css.simulation.resource.common.configuration.git.GitConfiguration;
import com.css.simulation.resource.feign.AlgoPlatformService;
import com.css.simulation.resource.feign.FileDownService;
import com.css.simulation.resource.feign.SchedulerService;
import com.css.simulation.resource.project.service.SimulationProjectService;
import com.css.simulation.resource.system.service.DictService;
import com.github.pagehelper.PageInfo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
public class AlgorithmServiceImpl implements AlgorithmService {

    AlgoPlatformService algoPlatformService;
    SchedulerService schedulerService;
    FileDownService fileDownService;
    AlgorithmMapper algorithmMapper;
    DictService dictService;
    GitConfiguration gitConfiguration;

    @Autowired
    private SimulationProjectService simulationProjectService;

    @Autowired
    public AlgorithmServiceImpl(AlgoPlatformService algoPlatformService,
                                SchedulerService schedulerService,
                                @Qualifier("com.css.simulation.resource.feign.FileDownService") FileDownService fileDownService,
                                AlgorithmMapper algorithmMapper,
                                DictService dictService,
                                GitConfiguration gitConfiguration
    ) {
        this.algoPlatformService = algoPlatformService;
        this.schedulerService = schedulerService;
        this.fileDownService = fileDownService;
        this.algorithmMapper = algorithmMapper;
        this.dictService = dictService;
        this.gitConfiguration = gitConfiguration;
    }

    //* -------------------------------- Comment --------------------------------


    @Override
    public ResponseBodyVO addOrUpdate(AlgorithmParameter param) {

        String currentUserId = AuthUtil.getCurrentUserId();
        param.setCreateUserId(currentUserId);
        List<AlgorithmPO> algorithmPOS = algorithmMapper.selectAlgorithmName(param);
        if (algorithmPOS != null && algorithmPOS.size() > 0) {
            return new ResponseBodyVO(ResponseBodyVO.Response.SERVER_FAILURE, "该算法名称已存在！");
        } else {
            String uploadMode = param.getUploadMode();
            if (DictConstants.FILE.equals(uploadMode)) {
                String minioPath = param.getMinioPath();
                ResponseBodyVO<String> checkRes = schedulerService.check(minioPath);
                if (ObjectUtil.isNull(checkRes)) {
                    return new ResponseBodyVO(ResponseBodyVO.Response.SERVER_FAILURE, "算法校验服务异常，请联系管理人员！");
                }
                if (!checkRes.isStatus()) {//校验失败
                    MinioParameter minioParameter = new MinioParameter();
                    minioParameter.setObjectName(minioPath);
                    fileDownService.remove(minioParameter);
                    return new ResponseBodyVO(ResponseBodyVO.Response.SERVER_FAILURE, "该算法经校验不可用，无法保存！");
                }
            }
            return doAddOrUpdate(param);
        }
    }

    private ResponseBodyVO doAddOrUpdate(AlgorithmParameter param) {
        if (isEmpty(param.getId())) {
            //添加
            String algorithmId = StringUtil.getRandomUUID();
            param.setId(algorithmId);
            String algorithmCode = StringUtil.getRandomCode();
            param.setAlgorithmCode(algorithmCode);
            AlgorithmPO po = setPo(param, false);
            int add = algorithmMapper.add(po);
            if (add > 0) {
                LogUtil.insert();
                return new ResponseBodyVO(ResponseBodyVO.Response.SUCCESS);
            } else {
                return new ResponseBodyVO(ResponseBodyVO.Response.SERVER_FAILURE, "添加失败");
            }
        } else {
            //编辑
            AlgorithmPO po = setPo(param, true);
            int add = algorithmMapper.update(po);
            if (add > 0) {
                LogUtil.update();
                return new ResponseBodyVO(ResponseBodyVO.Response.SUCCESS);
            } else {
                return new ResponseBodyVO(ResponseBodyVO.Response.SERVER_FAILURE, "编辑失败");
            }
        }

    }

    @Override
    public ResponseBodyVO selectAlgorithmList(AlgorithmParameter param) {
        param.setIsDeleted("0");
        param.setCreateUserId(AuthUtil.getCurrentUserId());
        PageUtil.setPageInfo(param);
        List<AlgorithmVO> vos = algorithmMapper.selectAlgorithmList(param);
        for (AlgorithmVO vo : vos) {
            vo.setValidationStatus(getDictName(DictConstants.VALIDATION_STATUS, vo.getValidationStatus()));
        }
        return new ResponseBodyVO<>(ResponseBodyVO.Response.SUCCESS, new PageInfo<>(vos));
    }

    @Override
    public ResponseBodyVO selectAlgoPlatformList(AlgorithmParameter param) {
        //获取参数
        String algorithmId = param.getAlgorithmCode();
        String algorithmName = param.getAlgorithmName();
        String description = param.getDescription();
        Integer page = param.getPageNum();
        Integer size = param.getPageSize();
        String sort = "algorithmId-desc";
        String urlParam = "";
        if (StringUtil.isNotEmpty(algorithmId)) {
            urlParam = urlParam + "&algorithmId=" + algorithmId;
        }
        if (StringUtil.isNotEmpty(algorithmName)) {
            urlParam = urlParam + "&algorithmName=" + algorithmName;
        }
        if (StringUtil.isNotEmpty(description)) {
            urlParam = urlParam + "&description=" + description;
        }
        if (StringUtil.isNotEmpty(page)) {
            urlParam = urlParam + "&page=" + page;
        }
        if (StringUtil.isNotEmpty(size)) {
            urlParam = urlParam + "&size=" + size;
        }
        urlParam = urlParam + "&sort=" + sort;

        ResponseBodyVO algorithmList = algoPlatformService.getAlgorithmList(urlParam);

        //解析数据
        Map jsonMap = JsonUtil.jsonToMap((String) algorithmList.getInfo());
        Map<String, Object> dataMap = (Map<String, Object>) jsonMap.get("data");
        List<Map<String, String>> contentList = (List<Map<String, String>>) dataMap.get("content");
        Integer totalElements = (Integer) dataMap.get("totalElements");
        List<AlgorithmVO> vos = new ArrayList<>();
        if (totalElements > 0) {
            for (Map<String, String> content : contentList) {
                AlgorithmVO vo = new AlgorithmVO();
                vo.setId(content.get("id"));
                vo.setAlgorithmCode(content.get("algorithmId"));
                vo.setAlgorithmName(content.get("algorithmName"));
                vo.setDescription(content.get("description"));
                vos.add(vo);
            }
        }
        PageInfo<AlgorithmVO> objectPageInfo = new PageInfo<>(vos);
        objectPageInfo.setTotal(totalElements);

        return new ResponseBodyVO<>(ResponseBodyVO.Response.SUCCESS, objectPageInfo);
    }


    @Override
    public ResponseBodyVO selectSharedAlgorithmList(AlgorithmParameter param) {
        param.setShare("1");
        PageUtil.setPageInfo(param);
        List<AlgorithmVO> vos = algorithmMapper.selectSharedAlgorithmList(param);
        for (AlgorithmVO vo : vos) {
            vo.setValidationStatus(getDictName(DictConstants.VALIDATION_STATUS, vo.getValidationStatus()));
        }
        return new ResponseBodyVO<>(ResponseBodyVO.Response.SUCCESS, new PageInfo<>(vos));
    }

    @Override
    public ResponseBodyVO shareAlgorithm(AlgorithmParameter param) {
        if (isEmpty(param.getId())) {
            return new ResponseBodyVO(ResponseBodyVO.Response.CLIENT_FAILURE);
        } else {
            String algorithmId = param.getId();
            AlgorithmPO po = algorithmMapper.selectDetailsById(algorithmId);
            List<AlgorithmPO> algorithmPOS = algorithmMapper.selectSharedAlgorithmName(po.getAlgorithmName());
            if (algorithmPOS != null && algorithmPOS.size() > 0) {
                return new ResponseBodyVO(ResponseBodyVO.Response.SERVER_FAILURE, "该算法名称已存在, 无法分享！");
            } else {
                po.setShare("1");
                po.setId(StringUtil.getRandomUUID());
                po.setAlgorithmCode(StringUtil.getRandomCode());
                int add = algorithmMapper.add(po);
                if (add > 0) {
                    LogUtil.share();
                    return new ResponseBodyVO(ResponseBodyVO.Response.SUCCESS, "分享成功");
                }
                return new ResponseBodyVO(ResponseBodyVO.Response.SERVER_FAILURE, "分享失败");

            }
        }
    }

    @Override
    public ResponseBodyVO<String> testConnection(AlgorithmParameter param) {
        try {
            JgitUtil.testConnection(param.getGitUrl().replace(gitConfiguration.getName(), gitConfiguration.getUrl()), param.getGitUserName(), param.getGitPassword());
            return new ResponseBodyVO<>(ResponseBodyVO.Response.SUCCESS, "连接成功");
        } catch (Exception e) {
            return new ResponseBodyVO<>(ResponseBodyVO.Response.SERVER_FAILURE, "连接失败！");
        }
    }

//    @Override
//    public String getGitVersion(String algorithmId) {
//        AlgorithmPO algorithmPO = algorithmMapper.selectDetailsById(algorithmId);
//        String gitUrl = algorithmPO.getGitUrl();
//        String gitUserName = algorithmPO.getGitUserName();
//        String gitPassword = algorithmPO.getGitPassword();
//        if (StringUtil.isEmpty(gitUrl)) {
//            return "";
//        }
//        //拼接get请求地址
////        String prefix = "https://api.github.com/repos/";
//        String prefix = "https://api.github.com/repos/";
//        String suffix = "/commits?page=1&per_page=1";
//        String gitInfo = gitUrl.substring(19, gitUrl.length() - 4);
//        String requestUrl = prefix + gitInfo + suffix;
//        try {
//            String result = HttpUtil.get(HttpUtil.getHttpClient(), HttpUtil.getRequestConfig(), requestUrl);
//            List<HashMap> hashMaps = JsonUtil.jsonToList(result, HashMap.class);
//            if (hashMaps.size() > 0) {
//                HashMap hashMap = hashMaps.get(0);
//                String version = hashMap.get("sha").toString();
//                return version;
//            }
//        } catch (Exception e) {
//            e.printStackTrace();
//        }
//        return StringUtil.getRandomCode();
//    }

    @Override
    public String getGitVersion(String algorithmId) {
        AlgorithmPO algorithmPO = algorithmMapper.selectDetailsById(algorithmId);
        String gitUrl = algorithmPO.getGitUrl();
        String replace = gitUrl.replace(gitConfiguration.getName(), gitConfiguration.getUrl());
        String gitUserName = algorithmPO.getGitUserName();
        String gitPassword = algorithmPO.getGitPassword();
        return GitLabUtil.getLastCommitTimestamp(replace, gitUserName, gitPassword) + "";
    }

    @Override
    public ResponseBodyVO<String> deleteById(AlgorithmParameter param) {
        if (isEmpty(param.getId())) {
            return new ResponseBodyVO<>(ResponseBodyVO.Response.CLIENT_FAILURE);
        }
        String algorithmId = param.getId();
        // 查询算法相关的正在运行的项目
        List<RunningProjectVO> runningProjectVos = algorithmMapper.selectRunningProject(algorithmId);
        if (runningProjectVos != null && runningProjectVos.size() > 0) {
            StringBuilder stringBuilder = new StringBuilder();
            for (RunningProjectVO vo : runningProjectVos) {
                // 删除算法将停止所有使用该算法的正在运行中的工作项目
                // 子项目
                SimulationManualProjectParam voparam = new SimulationManualProjectParam();
                voparam.setId(vo.getId());
                voparam.setNowRunState(DictConstants.PROJECT_TERMINATED);
                simulationProjectService.updateProjectNowRunState(voparam);
            }

            // 停用所有父项目
            List<RunningProjectVO> runningParentProjectVos = algorithmMapper.selectRunningProjectParent(algorithmId);
            if (runningParentProjectVos != null && runningParentProjectVos.size() > 0) {
                for (RunningProjectVO voParent : runningParentProjectVos) {
                    SimulationManualProjectParam voParamParent = new SimulationManualProjectParam();
                    stringBuilder.append(voParent.getProjectId()).append(",");
                    voParamParent.setId(voParent.getId());
                    voParamParent.setAutomaticRunState("1");
                    simulationProjectService.updateAutomaticRunState(voParamParent);
                }
            }
            String substring = stringBuilder.substring(0, stringBuilder.lastIndexOf(","));
        }
        algorithmMapper.deleteById(algorithmId);
        LogUtil.delete();
        return new ResponseBodyVO<>(ResponseBodyVO.Response.SUCCESS);

    }

    @Override
    public ResponseBodyVO<Map<String, String>> deleteCheck(AlgorithmParameter param) {
        if (isEmpty(param.getId())) {
            return new ResponseBodyVO<>(ResponseBodyVO.Response.CLIENT_FAILURE);
        } else {
            Map<String, String> map = new HashMap<>();
            String algorithmId = param.getId();
            List<RunningProjectVO> runningParentProjectVos = algorithmMapper.selectAutoProjectIdByAlgorithmId(algorithmId);
            if (runningParentProjectVos != null && runningParentProjectVos.size() > 0) {
                StringBuilder stringBuilder = new StringBuilder();
                for (RunningProjectVO voParent : runningParentProjectVos) {
                    stringBuilder.append(voParent.getProjectId()).append(",");
                }
                String substring = stringBuilder.substring(0, stringBuilder.lastIndexOf(","));
                map.put("substring", substring);
                return new ResponseBodyVO<>(ResponseBodyVO.Response.SUCCESS, "该算法已绑定 ID 为 " + substring + " 的自动运行项目，删除后该项目将停止自动运行。", map);
            } else {
                map.put("substring", "");
                return new ResponseBodyVO<>(ResponseBodyVO.Response.SUCCESS, "该算法未绑定自动运行项目。", map);
            }
        }
    }

    @Override
    public ResponseBodyVO selectDetailsById(String algorithmId) {
        if (isEmpty(algorithmId)) {
            return new ResponseBodyVO(ResponseBodyVO.Response.CLIENT_FAILURE);
        } else {
            AlgorithmPO po = algorithmMapper.selectDetailsById(algorithmId);
            po.setValidationStatus(getDictName(DictConstants.VALIDATION_STATUS, po.getValidationStatus()));
            return new ResponseBodyVO<>(ResponseBodyVO.Response.SUCCESS, po);
        }

    }


    private boolean isEmpty(String value) {
        if (value == null) {
            return true;
        }
        value = value.trim();
        if (value.length() == 0) {
            return true;
        }
        return false;
    }


    private AlgorithmPO setPo(AlgorithmParameter param, Boolean isEdit) {

        AlgorithmPO po = new AlgorithmPO();
        //设置po信息
        if (DictConstants.GIT.equals(param.getUploadMode())) { //使用仓库地址上传
            String gitUrl = param.getGitUrl();
            String gitUserName = param.getGitUserName();
            String gitPassword = param.getGitPassword();
            po.setGitUrl(gitUrl);
            po.setGitUserName(gitUserName);
            po.setGitPassword(gitPassword);
        } else {
            String minioPath = param.getMinioPath();
            String[] splits = minioPath.split("/");
            String fileName = splits[splits.length - 1];
            po.setMinioPath(param.getMinioPath());
            po.setFileName(fileName);
        }
        po.setAlgorithmCode(param.getAlgorithmCode());
        po.setAlgorithmName(param.getAlgorithmName());
        po.setDescription(param.getDescription());
        po.setUploadMode(param.getUploadMode());
        po.setValidationStatus(DictConstants.SUCCESS);
        po.setShare(DictConstants.NO);
        po.setId(param.getId());

        if (isEdit) {
            PoUtil.initUpdatePo(po);
        } else {
            PoUtil.initAddPo(po);
        }
        return po;

    }

    private String getDictName(String type, String code) {
        DictParam dictParam = new DictParam();
        dictParam.setDictTypes(type);
        Map<String, Map<String, String>> dictMapsByTypes = dictService.getDictMapsByTypes(dictParam);
        Map<String, String> stringStringMap = dictMapsByTypes.get(type);
        if (stringStringMap != null && stringStringMap.size() > 0) {
            return stringStringMap.get(code);
        }
        return "";
    }

    @Override
    public Map<String, String> selectDetailsBySy() {
        AlgorithmParameter param = new AlgorithmParameter();
        param.setUploadMode("1");
        param.setShare("0");
        param.setCreateUserId(AuthUtil.getCurrentUserId());
        int fileNum = algorithmMapper.selectDetailsBySy(param);
        param.setUploadMode("2");
        int pathNum = algorithmMapper.selectDetailsBySy(param);
        Map<String, String> map = new HashMap<>();
        map.put("fileNum", fileNum + "");
        map.put("pathNum", pathNum + "");
        return map;
    }

}
